import React, { useEffect, useMemo, useRef } from 'react';
import { Table, AugmentedNodeAttributes } from './components/Table';
import { SchemaAttribute } from '../../../schema';
import { VisualizationPropTypes, VISComponentType } from '../../common';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { GraphMetaData } from '@graphpolaris/shared/lib/data-access/statistics';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';

export type TableProps = {
  showBarplot: boolean;
  itemsPerPage: number;
  displayAttributes: string[];
  displayEntity: string;
};

const configuration: TableProps = {
  itemsPerPage: 10,
  showBarplot: true,
  displayAttributes: [],
  displayEntity: '',
};

export const TableVis = ({ data, schema, configuration }: VisualizationPropTypes) => {
  const ref = useRef<HTMLDivElement>(null);

  const attributesArray = useMemo<AugmentedNodeAttributes[]>(
    () =>
      data.nodes
        .filter((node) => node.label === configuration.displayEntity)
        .map((node) => {
          const types: SchemaAttribute[] =
            schema.nodes.find((n) => n.key === node.label)?.attributes?.attributes ??
            schema.edges.find((r) => r.key === node.label)?.attributes?.attributes ??
            [];

          return {
            attribute: node.attributes,
            type: Object.fromEntries(types.map((t) => [t.name, t.type])),
          };
        }),
    [data.nodes, configuration.displayEntity],
  );

  return (
    <div className="h-full w-full" ref={ref}>
      {attributesArray.length > 0 && (
        <Table
          data={attributesArray}
          itemsPerPage={configuration.itemsPerPage}
          showBarPlot={configuration.showBarplot}
          showAttributes={configuration.displayAttributes}
          selectedEntity={configuration.displayEntity}
        />
      )}
    </div>
  );
};

const TableSettings = ({
  configuration,
  graph,
  updateSettings,
}: {
  configuration: TableProps;
  graph: GraphMetaData;
  updateSettings: (val: any) => void;
}) => {
  useEffect(() => {
    if (graph && graph.nodes && graph.nodes.labels.length > 0) {
      updateSettings({ displayEntity: graph.nodes.labels[0] });
    }
  }, [graph]);

  const selectedNodeAttributes = useMemo(() => {
    if (configuration.displayEntity) {
      const nodeType = graph.nodes.types[configuration.displayEntity];
      if (nodeType && nodeType.attributes) {
        return Object.keys(nodeType.attributes);
      }
    }
    return [];
  }, [configuration.displayEntity, graph]);

  useEffect(() => {
    if (graph && graph.nodes && graph.nodes.labels.length > 0) {
      updateSettings({ displayAttributes: selectedNodeAttributes });
    }
  }, [selectedNodeAttributes]);

  return (
    <SettingsContainer>
      <Input
        type="dropdown"
        label="Select entity"
        value={configuration.displayEntity}
        onChange={(val) => updateSettings({ displayEntity: val })}
        options={graph.nodes.labels}
      />
      <Input
        type="boolean"
        label="Show barplot"
        value={configuration.showBarplot}
        onChange={(val) => updateSettings({ showBarplot: val })}
      />
      <Input
        type="dropdown"
        label="Items per page"
        value={configuration.itemsPerPage}
        onChange={(val) => updateSettings({ itemsPerPage: val })}
        options={[10, 25, 50, 100]}
      />
      <div>
        <span className="text-sm">Attributes to display</span>
        <div className="">
          <Input
            type="checkbox"
            value={configuration.displayAttributes}
            options={selectedNodeAttributes}
            onChange={(val: string[] | string) => {
              const updatedVal = Array.isArray(val) ? val : [val];
              updateSettings({ displayAttributes: updatedVal });
            }}
          />
        </div>
      </div>
    </SettingsContainer>
  );
};

export const TableComponent: VISComponentType = {
  displayName: 'TableVis',
  component: TableVis,
  settings: TableSettings,
  configuration: configuration,
};

export default TableComponent;
